/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2021 Marcus Britanicus (https://gitlab.com/marcusbritanicus)
 * Copyright (c) 2021 Abrar (https://gitlab.com/s96Abrar)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <QCoreApplication>
#include <QObject>
#include <QThread>
#include <QFile>
#include <QDebug>
#include <signal.h>

#include <signal.h>

#include "DataControlImpl.hpp"

/**
 * Timed IO
 * Sometimes, GTK apps cause problems while reading the data
 * they provide. In such cases, we should try to read data for
 * upto 1s, and then, quit. This QObject-derived class does
 * exactly what's written above.
 * Currently, we read the data in 4 batches of 250ms wait.
 */
PipeReader::PipeReader() {
    /**
     * In the current implementation,
     * we don't have any thing to initialize.
     * This behaviour can change at a later stage.
     */
}


QByteArray PipeReader::readFromPipe( int fd ) {
    QByteArray data;
    QFile      f;


    /** See if we can open the fd */
    if ( f.open( fd, QFile::ReadOnly, QFile::AutoCloseHandle ) ) {
        for ( int i = 0; i < 4; i++ ) {
            /** Wait for 250 ms for the data to become available */
            f.waitForReadyRead( 250 );

            /** Blindly assume that there is data to read */
            data.append( f.readAll() );

            /** Process the Application event queue */
            qApp->processEvents();
        }

        /** Close the file device and the pipe fd */
        f.close();
    }

    /** Return the data we have colelcted so far. */
    return data;
}
